/*
 * =====================================================================================
 *
 *       Filename:  main.c
 *
 *    Description:  main function
 *
 *        Version:  1.0
 *        Created:  2020-11-29 07:20:22 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Alvin Sukmadji 
 *   Organization:  University of Toronto
 *
 *   Note: This license is a modified BSD License, which contains the terms of noncommercial
 *   use.
 *   
 *   Copyright <2020> <Alvin Yonathan Sukmadji>
 *   
 *   Redistribution and use in source and binary forms, with or without modification, are
 *   permitted provided that the following conditions are met:
 *   
 *   1. Redistributions of source code must retain the above copyright notice, this list of
 *   conditions and the following disclaimer.
 *   
 *   2. Redistributions in binary form must reproduce the above copyright notice, this list
 *   of conditions and the following disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 *   
 *   3. Neither the name of the copyright holder nor the names of its contributors may be used
 *   to endorse or promote products derived from this software without specific prior written
 *   permission.
 *   
 *   4. This software may only be used for noncommercial purposes, including teaching and
 *   academic research at universities, colleges and other educational institutions, and
 *   personal non-profit purposes. Neither this software, whether in Source or Object form,
 *   nor any derivative work that is based on (or derived from) this software may be used for
 *   commercial purposes, for example, licensing or selling the software, or using the
 *   software with a purpose to procure a commercial gain.
 *   
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 *   EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 *   OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *   INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * =====================================================================================
 */

#include "common.h"
#include "zipper.h"
#include "pcg_variants.h"
#include "bch.h"

#define MAX_ERROR 20000

int main(int argc, char *argv[])
{
	if(argc < 4)
	{
		printf("usage: ./main config_filename chunk_size crossover_probability\n");
		exit(1);
	}
	ZIPPER z;
	pcg32f_random_t rng;
	int num_error = 0;
	long long int num_info_symbols = 0;
	int iter = 0;
	double p = atof(argv[3]);
	pcg32f_srandom_r(&rng, time(NULL)); // initialize rng seed

	zipper_setup(&z, argv[1], atoi(argv[2]));

	// print code and channel parameters
	printf("**************************************************************\n");
	printf("ZIPPER CODE AND SIMULATION PARAMETERS\n");
	printf("period = %d\n", z.period);
	printf("interleaver map:\n");
	for(int i = 0; i < z.period; i++)
	{
		printf("\trow %d: n = %d, k = %d, m = %d\n", i, z.n[i], z.k[i], z.m[i]);
	}
	printf("buffer length = %d\n", z.buffer_len);
	printf("chunk size = %d\n", z.chunk_size);
	printf("crossover probability = %.4e\n", p);
	printf("using constituent code BCH (%d, %d)\n", BCH_get_n(), BCH_get_k());
	printf("**************************************************************\n");
	printf("SIMULATION START:\n");
	while(num_error < MAX_ERROR) //terminate simulation once MAX_ERROR errors are collected
	{
		zipper_encode(&z, iter, &rng, p);
		num_error += zipper_decode(&z, iter, &num_info_symbols);
		if(iter%1000 == 0 && iter > 0) //print output message after every 1000 chunks
		{
			printf("%d chunks transmitted -- %.3e information symbols and %d errors collected\n", iter, 1.0*num_info_symbols, num_error);
		}
		iter++;
	}

	printf("SIMULATION FINISHED:\n");
	printf("%d chunks transmitted -- %lld information symbols and %d errors collected\n", iter, num_info_symbols, num_error);
	printf("BER = %.4e\n", 1.0*num_error/(num_info_symbols));

	zipper_cleanup(&z);
}